'use strict';

/**
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */

/**
 * Trigger a callback when 'this' image is loaded:
 * @param {Function} callback
 * @source https://stackoverflow.com/questions/3877027/jquery-callback-on-image-load-even-when-the-image-is-cached/3877079
 */
(function($)
{
    $.fn.imgLoad = function(callback)
	{
        return this.each(function()
		{
            if (callback)
			{
                if (this.complete || /*for IE 10-*/ $(this).height() > 0)
				{
                    callback.apply(this);
                }
                else
				{
                    $(this).on('load', function()
					{
                        callback.apply(this);
                    });
                }
            }
        });
    };
})(jQuery);

// Identifiant des fichiers sélectionnés en mode "grille".
var selected_group_id;

// Échape les caractères spéciaux pour la recherche.
function search_escape(str)
{
	str = str.replace(/[\-\[\]\/\{\}\(\)\*\+\?\.\\\^\$\|]/g, "\\$&");
	str = str.replace(/\s+/g, '\\s+')
	return str;
};

// Messages de rapport.
var timeout_report;
function report()
{
	if ($('#report').length)
	{
		timeout_report = setTimeout(function()
		{
			$('#report:not(#scan_report,.error)').hide();
		}, 3400);
	}
	if ($('#report').length)
	{
		var direction = $('#report').css('bottom') == '0px' ? 'down' : 'left';
	}
	$('#report:not(#scan_report) .report_msg').click(function()
	{
		$(this).hide('slide', { direction: direction }, 200);
	});
}

// Montre ou cache un élément selon son état.
function showhide(sel)
{
	if ($(sel).is(':hidden'))
	{
		$(sel).show();
	}
	else
	{
		$(sel).hide();
	}
}

// Objets : (dé)coche une case à cocher.
function select(e, type)
{
	if (e.prop('disabled'))
	{
		return;
	}
	var select = function(e)
	{
		if (!e.prop('checked'))
		{
			e.prop('checked', true);
			e.parents('.selectable_class').addClass('selected').removeClass('unselected');
		}
	};
	var unselect = function(e)
	{
		if (e.prop('checked'))
		{
			e.prop('checked', false);
			e.parents('.selectable_class').addClass('unselected').removeClass('selected');
		}
	};
	switch (type)
	{
		case 'all' :
			select(e);
			break;

		case 'invert' :
			if (e.prop('checked'))
			{
				unselect(e);
			}
			else
			{
				select(e);
			}
			break;

		case 'none' :
			unselect(e);
			break;
	}
	$('#select_num').text($('.selectable_class.selected').length);
}

// Selectionne tous les objets.
function select_all()
{
	$('.selectable').each(function()
	{
		select($(this), 'all');
	});
}

// Déselectionne tous les objets.
function select_none()
{
	$('.selectable').each(function()
	{
		select($(this), 'none');
	});
}

// Inverse la sélection des objets.
function select_invert()
{
	$('.selectable').each(function()
	{
		select($(this), 'invert');
	});
}

// Montre les details de tous les objets.
function show_all()
{
	if ($('#show_items').length)
	{
		var sel = $('#show_items :selected').val();
		$('.show_parts a' + (sel == 'obj_fold' ? '' : '[data-id^="' + sel + '"]'))
			.addClass('show');
		$('.' + sel).addClass('show');
		return;
	}
	$('.show_parts a,.obj_fold').addClass('show');
	$('.obj_mini_edit').show();
}

// Cache les details de tous les objets.
function hide_all()
{
	if ($('#show_items').length)
	{
		var sel = $('#show_items :selected').val();
		$('.show_parts a' + (sel == 'obj_fold' ? '' : '[data-id^="' + sel + '"]'))
			.removeClass('show');
		$('.' + sel).removeClass('show');
		return;
	}
	$('.show_parts a,.obj_fold').removeClass('show');
	$('.obj_mini_edit').hide();
}

// Liens externes.
$('body').on('click', 'a.ex', function()
{
	var url = $(this).attr('href')
	if (url.match(/^(ftp|https?):/))
	{
		$(this).attr('href', gallery_path + '/link.php?q=' + url);
	}
});

// Création d'un viewer pour l'affichage d'une image ou d'une vidéo.
function viewer(source)
{
	if (!source)
	{
		return;
	}

	page_scrollbar_hide();

	var html = '<div id="viewer">';
	var ext = source.replace(/^.+\.([^/]+)$/, '$1', source).toLowerCase();
	if (ext == 'mp4' || ext == 'webm')
	{
		html += '<video autoplay onloadstart="this.volume=0.5" controls>';
		html += '<source src="' + source + '&' + Math.random() + '" type="video/' + ext + '">';
		html += '</video>';
	}
	else
	{
		source += source.search(/\?/, source) == -1 ? '?' : '&';
		html += '<img src="' + source + Math.random() + '">';
	}
	html += '</div>';
	var resize = function()
	{
		var margin = parseInt($('#viewer').css('paddingLeft')) * 2;
		$('#viewer').css({
			'width': ($(window).width() - margin) + 'px',
			'height': ($(window).height() - margin) + 'px'
		});
	};
	$('main').append(html);
	resize();
	$(window).resize(function()
	{
		if ($('#viewer').is('div'))
		{
			resize();
		}
	});
	$('body').on('click', '#viewer', function()
	{
		$(this).remove();
		page_scrollbar_show();
	});
}



/**
 * Gestion de la scrollbar de la page.
 *
 */
var page_overflow;
function page_scrollbar_show()
{
	$('body').css(
	{
		'overflow': page_overflow.body,
		'padding': 0
	});

	if ($('#obj_grid').length && $('#obj_grid').hasClass('extend'))
	{
		$('#scrollbar_blank').remove();
		$('#obj_grid').css('right', 0);
	}
}
function page_scrollbar_hide()
{
	page_overflow =
	{
		body: $('body').css('overflow')
	};
	var width_before = $(window).width();
	$('body').css('overflow', 'hidden');
	var diff = ($(window).width() - width_before);
	$('body').css('padding-right', diff + 'px');

	if ($('#obj_grid').length && $('#obj_grid').hasClass('extend'))
	{
		$('main').append('<div id="scrollbar_blank"></div>');
		$('#scrollbar_blank').css({
			'position': 'absolute',
			'top': 0,
			'right': 0,
			'height': '200px',
			'width': diff + 'px',
			'background': 'white'
		});
		$('#obj_grid').css('right', diff + 'px');
	}
}



/**
 * Géolocalisation.
 *
 */
function Worldmap(prefix, callback)
{
	this.map;
	this.marker;
	this.markerActive = false;
	this.markerZoom = 12;
	this.tiles;

	var self = this;


	this.addMarker = function(latlng)
	{
		self.markerActive = true;
		self.marker = L.marker(latlng, {draggable: true, autoPan: true}).addTo(self.map);

		self.setLatlongFields();
		self.marker.on('drag', function()
		{
			self.setLatlongFields();
			self.resetCatList();
		});

		$('#' + prefix + '_marker')
			.attr('class', 'icon icon_marker_delete')
			.find('a').text(text_marker_delete);
	};

	this.deleteMarker = function()
	{
		if (!self.markerActive)
		{
			return;
		}

		self.markerActive = false;
		self.marker.remove();

		$('#' + prefix + '_latitude').val('');
		$('#' + prefix + '_longitude').val('');

		$('#' + prefix + '_marker')
			.attr('class', 'icon icon_marker_add')
			.find('a').text(text_marker_add);
	};

	// Retourne une coordonnée correctement formatée.
	this.getCoord = function(coord)
	{
		var regexp = /^(\d+(?:\.\d+)?)E\-(\d+)$/i;
		var match = regexp.exec(coord);
		if (match)
		{
			coord = '0.' + '0'.repeat(match[2] - 1) + match[1].replace('.', '');
		}
		return coord;
	};

	this.resetCatList = function()
	{
		$('#' + prefix + '_latitude, #' + prefix + '_longitude').prop('disabled', false);
		$('#' + prefix + '_cat_coords option').prop('selected', false);
		$('#' + prefix + '_cat_coords option[value="0"]').prop('selected', true);
	};

	this.setLatlongFields = function()
	{
		$('#' + prefix + '_latitude').val(self.getCoord(self.marker.getLatLng().lat));
		$('#' + prefix + '_longitude').val(self.getCoord(self.marker.getLatLng().lng));
	};

	this.setMarker = function(latlng)
	{
		self.deleteMarker();
		self.resetCatList();
		self.addMarker(latlng);
		self.map.setView(latlng, self.markerZoom);
	};

	this.setMap = function()
	{
		if (!$('#' + prefix + '_worldmap').length)
		{
			return;
		}

		self.map = L.map(prefix + '_worldmap',
		{
			center: [
				$('#' + prefix + '_worldmap').attr('data-lat'),
				$('#' + prefix + '_worldmap').attr('data-long')
			],
			zoom: $('#' + prefix + '_worldmap').attr('data-marker') == 1
				? self.markerZoom
				: $('#' + prefix + '_worldmap').attr('data-zoom')
		});
		self.tiles = L.tileLayer('https://{s}.tile.openstreetmap.fr/osmfr/{z}/{x}/{y}.png',
		{
			attribution: '<a href="//osm.org/copyright">OpenStreetMap</a>'
		}).addTo(self.map);

		// Moteur de recherche.
		// https://github.com/smeijer/leaflet-geosearch
		$.getScript(leafleft_search_js_file, function()
		{
			self.map.addControl(new GeoSearch.GeoSearchControl(
			{
				provider: new GeoSearch.OpenStreetMapProvider(),
				style: 'bar',
				autoClose: true,
				searchLabel: ' '
			}));
		});

		// Ajout du marqueur.
		if ($('#' + prefix + '_worldmap').attr('data-marker') == 1)
		{
			setTimeout(function(){self.addMarker(self.map.getCenter());}, 150);
		}

		if (typeof callback == 'function')
		{
			setTimeout(function(){callback();}, 150);
		}
	};

	// Ajout/suppresion du marqueur.
	$('body').on('click', '#' + prefix + '_marker', function()
	{
		// Suppression du marqueur.
		if (self.markerActive)
		{
			self.deleteMarker();
			self.resetCatList();
		}

		// Ajout du marqueur.
		else
		{
			self.addMarker(self.map.getCenter());
		}
	});

	// Coordonnées d'une catégorie déjà géolocalisée.
	$('body').on('change', '#' + prefix + '_cat_coords', function()
	{
		var option = $(this).find('option:selected');

		if (self.markerActive)
		{
			self.deleteMarker();
		}
		if (option.val() > 0)
		{
			$('#' + prefix + '_latitude, #' + prefix + '_longitude').prop('disabled', true);

			var lat = option.attr('data-lat');
			var lng = option.attr('data-long');
			self.addMarker([lat, lng]);
			self.map.setView([lat, lng], self.markerZoom);
		}
		else
		{
			$('#' + prefix + '_latitude, #' + prefix + '_longitude').prop('disabled', false);
		}
	});

	// On réactive les champs désactivés lors de l'envoi du formulaire.
	$('body').on('submit', '#' + prefix + '_form', function()
	{
		$('#' + prefix + '_latitude, #' + prefix + '_longitude').prop('disabled', false);
	});

	// Initialisation.
	if ($('#leafleft_css').length)
	{
		self.setMap();
	}
	else
	{
		$('head').append(
			'<link id="leafleft_css" rel="stylesheet" href="' + leafleft_css_file + '">' +
			'<link rel="stylesheet" href="' + leafleft_search_css_file + '">'
		);
		$.getScript(leafleft_js_file, self.setMap);
	}
}



/*
 * Listes flottantes.
 *
 */
jQuery(function($)
{
	if (!$('a[data-box-list]').length)
	{
		return;
	}

	// Élément de formulaire.
	var element_id;

	// Paramètres de la liste.
	var list_open = false;
	var list_type;

	// Ajax.
	var ajax_request;

	// Paramètres pour l'ajout de tags.
	var tags_item = [];

	// Paramètres pour la suppression de tags.
	var tags_delete = [];
	var tags_delete_flip = [];
	var tags_delete_selected = [];
	var timeout_loading;

	// Catégorie parente.
	var make_list_categories = function()
	{
		$('#box_list_outer h1 img').attr('src', style_path + '/icons/16x16/move.png');

		var list = $('#' + element_id).parents('p').find('select').html();
		list = list.replace(/<option([^>]+)>/g, '<li$1>').replace(/<\/option>/g, '</li>');

		$('#box_list').append('<ul>' + list + '</ul>');
	};

	// Ajout de tags.
	var make_list_tags_add = function()
	{
		$('#box_list_outer h1 img').attr('src', style_path + '/icons/16x16/tag.png');

		// Aucun tag ?
		if (!tags.length)
		{
			$('#box_list').append(
				'<p class="box_list_message">' + l10n_list_tags_empty + '</p>'
			);
			return;
		}

		// Fabrication de la liste.
		var html = '<ul>';
		for (var i in tags)
		{
			html += '<li data-tag-id="' + i + '"><span><span></span></span>'
				+ '<span>' + $('<textarea>').text(tags[i]).html() + '</span></li>';
		}
		html += '</ul>';
		$('#box_list').append(html);

		// Tags sélectionnés.
		tags_item = $('#' + element_id).val().split(',').filter(function(e)
		{
			return !e.match(/^\s*$/);
		});
		if (tags_item.length)
		{
			for (var i in tags_item)
			{
				tags_item[i] = tags_item[i].replace(/(^\s+|\s+$)/, '');
				if (tags_flip[tags_item[i]])
				{
					$('#box_list li[data-tag-id="' + tags_flip[tags_item[i]] + '"]')
						.addClass('selected');
				}
			}
		}
	};

	// Suppression de tags.
	var make_list_tags_delete = function()
	{
		$('#box_list_outer h1 img').attr('src', style_path + '/icons/16x16/tag.png');

		if (!selected_group_id.length)
		{
			$('#box_list').append(
				'<p class="box_list_message">' + l10n_items_none_selected + '</p>'
			);
			return;
		}

		// Icône de chargement.
		timeout_loading = setTimeout(function()
		{
			var html = '<div id="box_list_loading"><div class="loading"></div></div>';
			$('#box_list').append(html);
		}, 250);

		// Récupération des tags.
		ajax_request = $.post(
			gallery_path + '/ajax.php',
			{
				anticsrf: anticsrf,
				section: 'grid-get-tags-items',
				id: selected_group_id.join(',')
			},
			function(r)
			{
				if (typeof r != 'object' || r === null)
				{
					return;
				}
				switch (r.status)
				{
					case 'error' :
						alert('error: ' + r.message);
						break;

					case 'success' :
						anticsrf = r.anticsrf;
						tags_delete = r.tags;
						clearTimeout(timeout_loading);
						$('#box_list_loading').remove();

						if (tags_delete.length)
						{
							tags_delete_flip = [];
							for (var i in tags_delete)
							{
								tags_delete_flip[tags_delete[i]] = i;
							}

							// Fabrication de la liste.
							var html = '<ul>';
							for (var i in tags_delete)
							{
								html += '<li data-tag-id="' + i + '"><span><span></span></span>'
									+ '<span>' + $('<textarea>').text(tags_delete[i]).html()
									+ '</span></li>';
							}
							html += '</ul>';
							$('#box_list').append(html);

							// Tags sélectionnés.
							tags_delete_selected = $('#' + element_id).val().split(',')
							.filter(function(e)
							{
								return !e.match(/^\s*$/);
							});
							if (tags_delete_selected.length)
							{
								for (var i in tags_delete_selected)
								{
									tags_delete_selected[i] = tags_delete_selected[i]
										.replace(/(^\s+|\s+$)/, '');
									if (tags_delete_flip[tags_delete_selected[i]])
									{
										$('#box_list li[data-tag-id="'
											+ tags_delete_flip[tags_delete_selected[i]] + '"]')
											.addClass('selected');
									}
								}
							}
						}
						else
						{
							$('#box_list').append(
								'<p class="box_list_message">' + r.text_no_tag + '</p>'
							);
						}
						break;
				}
			},
			'json'
		);
	};

	// Tableau des tags.
	if (typeof tags !== 'undefined')
	{
		for (var i in tags)
		{
			tags[i] = $('<textarea>').html(tags[i]).text();
			tags_flip[tags[i]] = i;
		}
	}

	// Ouverture de la liste.
	$('body').on('click', 'a[data-box-list]', function()
	{
		list_open = true;
		list_type = $(this).attr('data-box-list');
		page_scrollbar_hide();
		var title = $(this).attr('data-box-title')
			? $(this).attr('data-box-title')
			: $(this).attr('title');

		var html = '<section data-box-list="' + list_type
			+ '" id="box_list_container"><div id="box_list_outer">'
			+ '<h1><img width="16" height="16"><span>' + title
			+ '</span></h1><div id="box_list_inner">'
			+ '<p id="box_list_search" class="field"><input type="text"></p>'
			+ '<div id="box_list"></div></div></div></section>';
		$('main').append(html);

		element_id = $(this).parents('p')
			.find(list_type == 'categories' ? 'select' : 'textarea').attr('id');

		if (list_type == 'categories')
		{
			make_list_categories();
		}
		if (list_type == 'tags_add')
		{
			make_list_tags_add();
		}
		if (list_type == 'tags_delete')
		{
			make_list_tags_delete();
		}

		$('#box_list_search input').focus();
	});

	// Fermeture de la liste.
	var close_list = function()
	{
		if (ajax_request)
		{
			ajax_request.abort();
		}
		list_open = false;
		$('#box_list_container').remove();
		page_scrollbar_show();
		$('#' + element_id).focus();
	};
	$('body').on('click', '#box_list_container', function(e)
	{
		close_list();
	});
	$('body').on('click', '#box_list_outer', function(e)
	{
		e.stopPropagation();
	});
	$(window).keyup(function(e)
	{
		if (list_open && e.keyCode == 27)
		{
			close_list();
		}
	});

	// Sélection d'un élément de la liste.
	$('body').on('click', '#box_list li', function()
	{
		if (list_type == 'categories' && !$(this).attr('disabled'))
		{
			$('#' + element_id).children('option').prop('selected', false);
			$('#' + element_id).children('option[value="'
				+ $(this).attr('value') + '"]').prop('selected', true);
			close_list();
		}

		if (list_type == 'tags_add')
		{
			var tag_name = tags[$(this).attr('data-tag-id')];
			if ($(this).hasClass('selected'))
			{
				$(this).removeClass('selected');
				tags_item = tags_item.filter(function(e)
				{
					return e != tag_name;
				});
			}
			else
			{
				$(this).addClass('selected');
				tags_item.push(tag_name);
			}
			$('#' + element_id).val(tags_item.join(', '));
		}

		if (list_type == 'tags_delete')
		{
			var tag_name = tags_delete[$(this).attr('data-tag-id')];
			if ($(this).hasClass('selected'))
			{
				$(this).removeClass('selected');
				tags_delete_selected = tags_delete_selected.filter(function(e)
				{
					return e != tag_name;
				});
			}
			else
			{
				$(this).addClass('selected');
				tags_delete_selected.push(tag_name);
			}
			$('#' + element_id).val(tags_delete_selected.join(', '));
		}
	});

	// Barre de recherche.
	$('body').on('input', '#box_list_search input', function(e)
	{
		$('#box_list li:not([disabled])').hide();
		if (list_type == 'categories')
		{
			var regexp = new RegExp(search_escape($(this).val()), 'i');
			$('#box_list li').each(function()
			{
				if (regexp.test($(this).text()))
				{
					$(this).show();
				}
			});
		}
		else
		{
			var regexp = new RegExp('^' + search_escape($(this).val()), 'i');
			$('#box_list li').each(function()
			{
				if (regexp.test($(this).find('span:last-child').text()))
				{
					$(this).show();
				}
			});
		}
	});
});



// Général.
jQuery(function($)
{
	report();

	// Indique que Javascript est activé.
	$('html.nojs').removeClass('nojs').addClass('js');

	// Changement de page.
	$('body:not(.grid) .pages form').submit(function()
	{
		var page = $(this).find('input[name="page"]').val();
		if (page.match(/^\d{1,12}$/) && !page.match(/^0/)
		&& parseInt(page) <= parseInt($(this).find('.count').text()))
		{
			window.location.href = '?q=' + q_pageless + '/page/' + page;
		}
		return false;
	});

	// Boutons de sélection de fichier.
	$('input[type="file"]').change(function(e)
	{
		if (e.target.value)
		{
			var filename = e.target.value.replace(/^.*?([^/\\]+)$/, '$1');
			var id = $(this).attr('id');
			$('#' + id + ' + label').text(filename);
		}
	});

	// Liens "viewer".
	$('body').on('click', '.viewer', function()
	{
		viewer($(this).attr('href'));
		return false;
	});

	// Lien de déconnexion.
	$('#user_logout').click(function()
	{
		$.post(
			gallery_path + '/ajax.php',
			{
				anticsrf: anticsrf,
				section: 'logout',
				from_admin: 1
			},
			function(r)
			{
				if (typeof r != 'object' || r === null)
				{
					return;
				}
				switch (r.status)
				{
					case 'error' :
						alert('error: ' + r.message);
						break;

					case 'success' :
						window.location = r.redirect;
						break;
				}
			},
			'json'
		);
		return false;
	});

	// Supprime la désactivation des boutons de formulaire pour
	// forcer leur utilisation seulement si Javascript est activé.
	$('input[type="submit"].js_required').prop('disabled', false);

	// Confirmation de suppression.
	$('input[type="submit"].js_required').click(function()
	{
		if ($(this).attr('name') == 'delete'
		|| $('#selection_action option[value="delete"]').prop('selected'))
		{
			return confirm(confirm_delete);
		}
	});

	// Pour la sélection, on affiche l'élément secondaire selon le type d'action choisi.
	$('#selection_action').change(function()
	{
		$(this).find('option[data-id]').each(function()
		{
			if ($(this).prop('selected'))
			{
				$('#' + $(this).attr('data-id')).css('display', 'inline-block');
			}
			else
			{
				$('#' + $(this).attr('data-id')).css('display', 'none');
			}
		});
	});

	// Affichage d'éléments de la page courante.
	$('.show_parts').click(function()
	{
		var id = '#' + $(this).find('a').attr('data-id');
		var obj_id = id.replace(/^.*_(\d+)$/, '$1');
		if ($(id).is(':hidden'))
		{
			$('#obj_' + obj_id  + ' .obj_fold').removeClass('show');
			$('#obj_' + obj_id  + ' a.show')
				.css('outline', $(this).find('a').attr('data-outline'))
				.removeClass('show');
			$(this).find('a')
				.attr('data-outline', $(this).find('a').css('outline'))
				.css('outline', 'none')
				.addClass('show');
			$(id).addClass('show');
			$(id + ' p:visible:first input').focus();
		}
		else
		{
			$(this).find('a')
				.css('outline', $(this).find('a').attr('data-outline'))
				.removeClass('show');
			$(id).removeClass('show');
		}
	});

	// Affichage d'outils (options, recherche, etc.).
	$('.show_tool').click(function()
	{
		$('.show_tool a').removeClass('show');
		var id = '#' + $(this).find('a').attr('data-id');
		if ($(id).is(':hidden'))
		{
			$('.tool').not($(id)).hide();
			$(id).show();
			$(this).find('a').addClass('show');
			$(id + ' .focus').focus();
		}
		else
		{
			$(id).hide();
		}
	});

	// Sélection d'objets.
	{
		// Initialisation.
		$('.selectable').prop('checked', false);

		// Case à cocher.
		$('.selectable').click(function()
		{
			select($(this), 'invert');
		});

		// Zone de sélection.
		$('.selectable_zone').click(function()
		{
			select($(this).find('.selectable'), 'invert');
		});
	}

	// Réinitialisation des dates.
	$('.date_reset').click(function()
	{
		$(this).parents('.field').find('.date_title').prop('selected', true);
	});

	// Gestion de l'affichage du menu.
	$('.js_menu_link').click(function(e)
	{
		if ($('#menu').is(':hidden'))
		{
			$('.js_box_inner').fadeOut(150);

			$('#menu').show('slide', { direction: 'left' }, 150, function()
			{
				$(window).keyup(function(e)
				{
					$(this).off('keyup');
					if (e.keyCode == 27)
					{
						$('#menu').hide('slide', { direction: 'left' }, 150);
					}
				});
				$(window).click(function()
				{
					if (!$('#header_links > img').is(':visible'))
					{
						$(this).off('click');
						$('#menu').hide('slide', { direction: 'left' }, 150);
					}
				});
				$('#menu').click(function(e)
				{
					$(this).off('click');
					e.stopPropagation();
				});
			});
		}
		else
		{
			$('#menu').hide('slide', { direction: 'left' }, 150);
		}
		e.stopPropagation();
	});

	// Réinitialisation des formulaires.
	$('form').trigger('reset');

	// Mode d'affichage et affichage en grille.
	$('#display a:not(.current)').click(function()
	{
		$('input[name="display"]').val($(this).attr('id').replace('display_', ''));
		$('#display form').submit();
	});
	if ($('#obj_grid').is('div'))
	{
		new Grid($('#obj_grid').attr('data-section'));
	}

	// Autoclicks et focus sur les champs à erreur (doit être en dernier).
	$('.autoclick').click();
	$('.autofocus').focus();
	$('.field_error input,.field_error textarea').focus();

	// Géolocalisation.
	if ($('#geolocation_container').length)
	{
		new Worldmap('geolocation');
	}

	// Captures vidéos.
	if (typeof captures !== 'undefined')
	{
		videocaptures(captures);
	}
});



/**
 * Page "Thèmes".
 *
 */
jQuery(function($)
{
	if (!$('#themes').length)
	{
		return;
	}

	$('.theme img, .theme_button').click(function()
	{
		$('.theme').removeClass('selected');
		$('.theme input[type="radio"]').prop('checked', false);
		$(this).parents('.theme').addClass('selected');
		$(this).parents('.theme').find('input[type="radio"]').prop('checked', true);
	});
});



/**
 * Page "Carte du monde".
 *
 */
jQuery(function($)
{
	if (!$('#section_page_worldmap').length)
	{
		return;
	}

	var geolocation = new Worldmap('geolocation');

	$('#geolocation_form input[type="submit"]').click(function()
	{
		var lat = geolocation.map.getCenter().lat;
		var lng = geolocation.map.getCenter().lng;
		var zoom = geolocation.map.getZoom();
		$(this).append(
			'<input name="latitude" type="hidden" value="' + lat + '">' +
			'<input name="longitude" type="hidden" value="' + lng + '">' +
			'<input name="zoom" type="hidden" value="' + zoom + '">'
		);
	});
});



/**
 * Objets "mini" de la partie "Réglages".
 *
 */
jQuery(function($)
{
	if (!$('#obj_mini_form').is('form'))
	{
		return;
	}

	// Actions.
	$('body').on('click', '.obj_mini_action .icon', function()
	{
		switch ($(this).attr('data-action'))
		{
			case 'delete' :
				$(this).parents('.obj_mini').remove();
				break;

			case 'edit' :
				var edit = $(this).parents('.obj_mini').find('.obj_mini_edit');
				if (edit.is(':hidden'))
				{
					edit.slideDown(250, function()
					{
						edit.find('p:first input').focus();
					});
				}
				else
				{
					edit.slideUp(250);
				}
				break;
		}
	});

	// Tri.
	$('#obj_mini_form').sortable(
	{
		axis: "y",
		containment: 'main',
		items: '.obj_mini',
		handle: '.obj_mini_sortable',
		opacity: 0.5
	});
	$('#obj_mini_form').submit(function()
	{
		$('input[name="serial"]').val($(this).sortable('serialize'));
	});

	// Liens externes : ajout d'un lien.
	$('#link_add').click(function()
	{
		$('.obj_mini_edit').hide();
		var n = 0;
		while ($('#i_' + n).is('div'))
		{
			n++;
		}
		$('#obj_mini_form > div').append(
			link_html.replace(/{style_path}/g, style_path).replace(/{id}/g, n)
		);
		$('#i_' + n).find('span[data-action="edit"] a').click();
	});
});



/**
 * Aide contextuelle.
 *
 */
jQuery(function($)
{
	if (!$('#help').is('div'))
	{
		return;
	}
	var html_container = function(item, id)
	{
		var content = $('*[data-help-content="' + item + '"]').html();
		var title = $('*[data-help-content="' + item + '"]').find('h1').text();
		return '<section class="help_context" data-help-id="' + id + '">'
				+ '<div class="help_title">'
					+ '<h1><span class="icon icon_help">' + title + '</span></h1>'
					+ '<span class="help_close"></span>'
				+ '</div>'
				+ '<div class="help_content">' + content + '</div>'
			+ '</section>';
	};
	var html_link = function(id)
	{
		return '<a data-help-link="' + id + '" title="' + help_text_link
			+ '"class="help_link" href="javascript:;"><img width="16" height="16" src="'
			+ style_path + '/icons/16x16/help-link.png"></a>';
	};
	$('*[data-help-icon]').each(function()
	{
		var item = $(this).attr('data-help-icon');
		var i = 1;
		var id = item + '_' + i;
		while ($('section[data-help-id="' + id + '"]').length)
		{
			i++;
			id = item + '_' + i;
		}
		$('*[data-help-container="' + item + '"]:eq(' + (i - 1) + ')')
			.after(html_container(item, id));
		$('*[data-help-icon="' + item + '"]:eq(' + (i - 1) + ')')
			.after(html_link(id));
	});
	$('body').on('click', '.help_link', function()
	{
		var item = $('*[data-help-id="' + $(this).attr('data-help-link') + '"]');
		if (item.is(':hidden'))
		{
			item.show();
		}
		else
		{
			item.hide();
		}
	});
	$('body').on('click', '.help_context .help_close', function()
	{
		var item = $(this).parents('.help_context').attr('data-help-id');
		$('*[data-help-id="' + item + '"]').hide();
	});
});



/**
 * Aide contextuelle : balises HTML.
 *
 */
jQuery(function($)
{
	if (!$('.html_inner').length)
	{
		return;
	}

	$('.html_inner > span').click(function()
	{
		var parent = $(this).parent();
		var textarea = parent.find('textarea');

		$('.html_inner').removeClass('active');
		$('.html_inner textarea').css({
			'min-height': 'auto',
			'visibility': 'visible'
		});

		if (parent.find('#help_textarea_html').length)
		{
			$('#help_textarea_html').remove();
			textarea.focus();
		}

		else
		{
			$('#help_textarea_html').remove();
			textarea.before(help_html);

			var width = textarea.outerWidth();
			$('#help_textarea_html').css(
			{
				'width': width + 'px',
				'max-width': width + 'px'
			});

			var height = $('#help_textarea_html').outerHeight();
			var width = textarea.width();
			textarea.css({
				'width': width + 'px',
				'min-width': width + 'px',
				'max-width': width + 'px',
				'min-height': (height - 10) + 'px',
				'visibility': 'hidden'
			});

			var title_height = $('#help_textarea_html .help_title').outerHeight();
			var margin = $('#obj_panel_edit').length ? 17 : 7;
			$('#help_textarea_html .help_content').css(
			{
				'height': (textarea.height() - margin - title_height) + 'px'
			});

			parent.addClass('active');
		}
	});
});



/**
 * Tableau de bord.
 *
 */
jQuery(function($)
{
	if (!$('#section_dashboard').length)
	{
		return;
	}

	// Navigation entre éléments d'un bloc.
	$(this).find('.dashboard_nav a').click(function()
	{
		var prev_or_next = $(this).attr('class').replace('dashboard_', '');
		var nb_items = $(this).parents('.dashboard_bloc')
			.find('.' + $(this).attr('rel')).length;
		var current = $(this).parents('.dashboard_bloc')
			.find('.' + $(this).attr('rel') + ':visible');
		var current_n = current.attr('id').replace($(this).attr('rel') + '_', '');
		var current_n_new = (prev_or_next == 'next' ?
				parseInt(current_n) + 1 : parseInt(current_n) - 1);

		// Élément courant.
		current.css('display', 'none');
		$(this).parents('.dashboard_bloc')
			.find('#' + $(this).attr('rel') + '_' + current_n_new).css('display', 'flex');

		// Position courante.
		$(this).parents('.dashboard_bloc').find('.dashboard_nav span')
			.text(current_n_new + '/' + nb_items);

		// Liens de navigation.
		if (current_n == 1)
		{
			$(this).parents('.dashboard_bloc').find('.dashboard_prev')
				.css('visibility', 'visible');
		}
		if (current_n == 2 && prev_or_next == 'prev')
		{
			$(this).parents('.dashboard_bloc').find('.dashboard_prev')
				.css('visibility','hidden');
		}
		if (parseInt(current_n) + 1 == nb_items && prev_or_next == 'next')
		{
			$(this).parents('.dashboard_bloc').find('.dashboard_next')
				.css('visibility', 'hidden');
		}
		if (current_n == nb_items)
		{
			$(this).parents('.dashboard_bloc').find('.dashboard_next')
				.css('visibility', 'visible');
		}
	});
});



/**
 * Boîtes flottantes.
 *
 */
jQuery(function($)
{
	// Listes "Parcourir".
	if ($('#browse').is('div'))
	{
		// Dimensions.
		var resize = function()
		{
			var available_height = $(window).height();
			var top = Math.round($('#browse_link')[0].getBoundingClientRect().bottom);
			var max_height = available_height - top - 100;
			max_height = max_height > 600 ? 600 : max_height;
			$('#browse_inner > div').css('maxHeight', max_height + 'px');
		};
		resize();
		$(window).resize(function(){resize();});

		// Recherche.
		$('#browse_search input').val('');
		$('#browse_search').on('input', function(e)
		{
			$('#browse_inner').css('width', $('#browse_inner').width() + 'px');

			var val = $(this).children('input').val();
			var regexp = new RegExp(search_escape(val), 'i');
			$('#browse li').hide();
			$('#browse li').each(function()
			{
				if (regexp.test($(this).find('span').text()))
				{
					$(this).show();
				}
			});
		});
	}

	// Gestion de l'affichage des boîtes.
	var timeout_box = [];
	var anim_speed = 200;
	var z_index_change = false;
	var close_box = function(box_id)
	{
		z_index_change = true;
		$('#' + box_id + ' .js_box_inner').fadeOut(anim_speed, function()
		{
			if (z_index_change)
			{
				$('.js_box').css('z-index', 4);
			}
		});
		$('#' + box_id).off('keyup');
	};
	var hide_box = function(box_id)
	{
		if ($('#' + box_id + ' .js_box_inner').is(':hidden')
		 || $('#' + box_id + ' .js_box_link').is(':hidden')
		 || $('#' + box_id + ' .js_box_inner input:first').is(':focus'))
		{
			return;
		}
		clear_box(box_id);
		timeout_box[box_id] = setTimeout(function(){close_box(box_id);}, 500);
	};
	var clear_box = function(box_id)
	{
		clearTimeout(timeout_box[box_id]);
	};
	$('.js_box_inner')
		.bind('mouseenter', function() { clear_box($(this).parents('.js_box').attr('id')) })
		.bind('mouseleave', function() { hide_box($(this).parents('.js_box').attr('id')) });
	$('.js_box_link')
		.bind('mouseenter', function() { clear_box($(this).parents('.js_box').attr('id')) })
		.bind('mouseleave', function() { hide_box($(this).parents('.js_box').attr('id')); });
	$('.js_box_link,.js_box_title').click(function()
	{
		var box_id = $(this).parents('.js_box').attr('id');
		z_index_change = false;
		for (var bi in timeout_box)
		{
			clearTimeout(timeout_box[bi]);
		}
		if ($('#' + box_id + ' .js_box_inner').is(':hidden'))
		{
			// Fermeture de toutes les boîtes.
			$('.js_box_inner').hide();

			// Si le menu est ouvert, on le ferme.
			if (!$('#menu .js_menu_link').is(':hidden'))
			{
				$('#menu .js_menu_link').click();
			}

			// On place la boîte courante devant toutes les autres.
			$('.js_box').css('z-index', 3);
			$('#' + box_id + '.js_box').css('z-index', 4);

			// Ouverture de la boîte.
			$('#' + box_id + ' .js_box_inner').fadeIn(anim_speed);
			$('#' + box_id + ' .js_box_inner input:first').focus();

			// Evénements pour la fermeture de la boîte.
			$(window).click(function() { close_box(box_id); $(this).off('click'); });
			$('#' + box_id).click(function(e) { e.stopPropagation(); });
			$('#' + box_id).keyup(function(e) { if (e.keyCode == 27) { close_box(box_id); } });
		}
		else
		{
			close_box(box_id);
		}
	});
});



/*
 * Section "Albums".
 *
 */
jQuery(function($)
{
	if ($('body').attr('id') != 'section_category'
	 && $('body').attr('id') != 'section_category_files'
	 && $('body').attr('id') != 'section_album')
	{
		return;
	}

	// Envoi de fichiers.
	if ($('#upload').is('div'))
	{
		new Upload('upload_', upload_options);
	}

	// Moteur de recherche.
	if ($('#search_categories').is('input'))
	{
		if ($('#search_categories').is(':checked'))
		{
			$('.search_items_fields').hide();
		}
		else
		{
			$('.search_categories_fields').hide();
		}
		$('#search_categories').click(function()
		{
			$('.search_items_fields').hide();
			$('.search_categories_fields').show();
		});
		$('#search_items').click(function()
		{
			$('.search_categories_fields').hide();
			$('.search_items_fields').show();
		});
	}
});



/*
 * Section Tags.
 *
 */
jQuery(function($)
{
	if (!$('#section_tags').length)
	{
		return;
	}

	$('#new_tags .icon_add').click(function()
	{
		var html = $('<input>').append($('#new_tags_fields input:last-child').clone()).html();
		$('#new_tags_fields').append(html.repeat(4));
	});
});



/*
 * Section Utilisateurs / Groupes / Permissions d'accès aux catégories.
 *
 */
jQuery(function($)
{
	if ($('body').attr('id') != 'section_group_categories')
	{
		return;
	}

	// Type de liste sélectionnée.
	$('#blacklist,#whitelist').each(function()
	{
		if ($(this).prop('checked'))
		{
			$(this).parents('.group_categories').addClass('select');
		}
	});

	// Modifie le style du type de liste sélectionné.
	$('#blacklist,#whitelist').click(function()
	{
		$('.group_categories').removeClass('select');
		$(this).parents('.group_categories').addClass('select');
	});

	// Déplie et replie des catégories.
	$('li > span a').click(function()
	{
		if (!$(this).parents('.group_categories').hasClass('select'))
		{
			return;
		}
		if ($(this).parent().hasClass('fold'))
		{
			$(this).parent().children('span + a + ul').css('display', 'block');
			$(this).parent().removeClass('fold');
			$(this).text('[-]');
		}
		else
		{
			$(this).parent().children('span + a + ul').css('display', 'none');
			$(this).parent().addClass('fold');
			$(this).text('[+]');
		}
	});

	// Permissions de la liste noire.
	$('#group_blacklist .perm').click(function()
	{
		if (!$(this).parents('.group_categories').hasClass('select'))
		{
			return;
		}
		if ($(this).hasClass('allowed'))
		{
			$(this).parent().find('.perm').each(function()
			{
				$(this)
					.removeClass('allowed')
					.removeClass('forbidden_child')
					.addClass('forbidden')
					.addClass('by_parent');
			});
			$(this).parents().each(function()
			{
				$(this).children('.perm')
					.addClass('forbidden_child');
			});
			$(this).removeClass('by_parent').removeClass('forbidden_child');
		}
		else if ($(this).hasClass('forbidden') && !$(this).hasClass('by_parent'))
		{
			$(this).parent().find('.perm').each(function()
			{
				$(this)
					.removeClass('forbidden')
					.removeClass('by_parent')
					.addClass('allowed');
			});
			$(this).parents().each(function()
			{
				if (!$(this).find('ul .perm').hasClass('forbidden'))
				{
					$(this).children('.perm')
						.removeClass('forbidden_child');
				}
			});
		}
	});

	// Permissions de la liste blanche.
	$('#group_whitelist .perm').click(function()
	{
		if (!$(this).parents('.group_categories').hasClass('select'))
		{
			return;
		}
		if ($(this).hasClass('forbidden'))
		{
			$(this).parent().find('.perm').each(function()
			{
				$(this)
					.removeClass('forbidden')
					.removeClass('allowed_child')
					.addClass('allowed')
					.addClass('by_parent');
			});
			$(this).parents().each(function()
			{
				$(this).children('.perm')
					.addClass('allowed_child');
			});
			$(this).removeClass('by_parent').removeClass('allowed_child');
		}
		else if ($(this).hasClass('allowed') && !$(this).hasClass('by_parent'))
		{
			$(this).parent().find('.perm').each(function()
			{
				$(this)
					.removeClass('allowed')
					.removeClass('by_parent')
					.addClass('forbidden');
			});
			$(this).parents().each(function()
			{
				if (!$(this).find('ul .perm').hasClass('allowed'))
				{
					$(this).children('.perm')
						.removeClass('allowed_child');
				}
			});
		}
	});

	// Envoi du formulaire.
	$('form#group_edit').submit(function()
	{
		// Nouvelle liste noire.
		var blacklist = '';
		$('#group_blacklist a.forbidden:not(.by_parent)').each(function()
		{
			blacklist += (blacklist != '' ? ',' : '') + $(this).attr('id').replace('b', '');
		});
		$('input[name="blacklist"]').val(blacklist);

		// Nouvelle liste blanche.
		var whitelist = '';
		$('#group_whitelist a.allowed:not(.by_parent):not(.allowed_child)').each(function()
		{
			whitelist += (whitelist != '' ? ',' : '') + $(this).attr('id').replace('w', '');
		});
		$('input[name="whitelist"]').val(whitelist);
	});
});



/*
 * Section "Groupes", "Fonctionnalités".
 *
 */
jQuery(function($)
{
	if (!$('#section_group_features').length)
	{
		return;
	}

	$('#group_account input').click(function()
	{
		if ($(this).val() == 1)
		{
			$('#group_permissions').hide();
		}
		else
		{
			$('#group_permissions').show();
		}
	});
});



/*
 * Section "Albums", tri des fichiers et des catégories.
 *
 */
jQuery(function($)
{
	if (!$('#section_album_sort').length
	 && !$('#section_category_sort').length)
	{
		return;
	}

	$('#thumbs_sort')
		.sortable(
		{
			containment: 'main',
			items: 'dl',
			opacity: 0.5
		})
		.submit(function()
		{
			$('input[name="serial"]').val($('#thumbs_sort').sortable('serialize'));
		});
});



/*
 * Section "Albums", nouvelle vignette de catégorie.
 *
 */
jQuery(function($)
{
	if (!$('#thumb_new_cat').length)
	{
		return;
	}

	$('.std_thumbs dt').click(function()
	{
		$('#thumb_id').val($(this).parents('dl').attr('data-id'));
		$('#thumb_new_cat').submit();
	});
});



/*
 * Section "Albums", capture d'image pour les vidéos.
 *
 */
jQuery(function($)
{
	if (!$('#thumb_video').length)
	{
		return;
	}

	var canplay = false;
	$('#thumb_video video').on('canplaythrough', function()
	{
		if (canplay)
		{
			return;
		}
		canplay = true;

		var video = $(this)[0];
		var width = video.videoWidth;
		var height = video.videoHeight;
		var duration = Math.round(video.duration);

		if (width < 1)
		{
			return;
		}

		$('#thumb_video_getcapture').click(function()
		{
			var canvas = document.createElement('canvas');
			var maxsize = 1920;
			canvas.width = width;
			canvas.height = height;

			if (width > maxsize || height > maxsize)
			{
				if (width < height)
				{
					canvas.width = Math.round((maxsize / height) * width);
					canvas.height = maxsize;
				}
				else
				{
					canvas.height = Math.round((maxsize / width) * height);
					canvas.width = maxsize;
				}
			}
			canvas.getContext('2d').drawImage(video, 0, 0, canvas.width, canvas.height);

			var canvas_data = canvas.toDataURL('image/jpeg');

			$('#thumb_video_capture').attr('src', canvas_data);
			$('input[name="video_width"]').val(width);
			$('input[name="video_height"]').val(height);
			$('input[name="video_duration"]').val(duration);
			$('input[name="video_capture"]').val(
				canvas_data.replace(/^data:image\/jpeg;base64,/, '')
			);
		});
	});
});



/*
 * Section "Albums", édition des vignettes.
 *
 */
jQuery(function($)
{
	if (!$('#section_item_thumb').length
	&& !$('#section_album_thumb').length && !$('#section_category_thumb').length)
	{
		return;
	}

	// Rognage.
	$('#thumb_crop img').imgLoad(function()
	{
		var max_width = 800;
		var max_height = 600;
		var ratio;

		var set_image_size = function(m_width)
		{
			var image_width = parseInt($('#thumb_crop img').attr('data-width'));
			var image_height = parseInt($('#thumb_crop img').attr('data-height'));

			var preview_width = $('#thumb_crop img').width();
			var preview_height = $('#thumb_crop img').height();

			var r_width = image_width / m_width;
			var r_height = image_height / max_height;

			if ((image_width > m_width) && (r_width >= r_height))
			{
				image_width = m_width;
				image_height = Math.round(image_height / r_width);
			}
			else if ((image_height > max_height) && (r_height >= r_width))
			{
				image_width = Math.round(image_width / r_height);
				image_height = max_height;
			}

			$('#thumb_crop img').height(image_height).width(image_width).show();

			ratio = parseInt($('#thumb_crop img').attr('data-width'))
				/ $('#thumb_crop img').width();

			return {'w': preview_width / image_width, 'h': preview_height / image_height};
		};

		var resize = function()
		{
			$('#thumb_crop img').hide();
			var scrollbar_width = 20;
			var container_width = Math.round($('#thumb_crop_container').width())
				- scrollbar_width;
			if (container_width > max_width)
			{
				container_width = max_width;
			}
			$('#thumb_crop img').show();

			return set_image_size(container_width);
		}

		resize();

		var crop = $('#thumb_crop img').imgAreaSelect(
		{
			handles: true,
			instance: true,
			minHeight: 50,
			minWidth: 50,
			persistent: true,
			x1: Math.round(parseInt($('#thumb_crop img').attr('data-selection-x')) / ratio),
			y1: Math.round(parseInt($('#thumb_crop img').attr('data-selection-y')) / ratio),
			x2: Math.round((parseInt($('#thumb_crop img').attr('data-selection-x'))
			  + parseInt($('#thumb_crop img').attr('data-selection-w'))) / ratio),
			y2: Math.round((parseInt($('#thumb_crop img').attr('data-selection-y'))
			  + parseInt($('#thumb_crop img').attr('data-selection-h'))) / ratio)
		});

		$(window).resize(function()
		{
			var r = resize();
			var s = crop.getSelection();

			crop.setSelection(s.x1 / r.w, s.y1 / r.h, s.x2 / r.w, s.y2 / r.h);
			crop.update();
		});

		$('#thumb_crop_select_all').click(function()
		{
			crop.setSelection(0, 0, $('#thumb_crop img').width(),
				$('#thumb_crop img').height());
			$('#thumb_crop_free').click();
		});

		$('#thumb_crop_free').click(function()
		{
			crop.setOptions(
			{
				aspectRatio: '',
				handles: true
			});
			crop.update();
		});

		$('#thumb_crop_square').click(function()
		{
			var height = $('#thumb_crop img').height();
			var width = $('#thumb_crop img').width();
			var x1 = 0;
			var y1 = 0;
			var x2 = 0;
			var y2 = 0;
			if (width > height)
			{
				x1 = Math.round((width - height) / 2);
				x2 = x1 + height;
				y2 = height;
			}
			else
			{
				y1 = Math.round((height - width) / 2);
				y2 = y1 + width;
				x2 = width;
			}
			crop.setSelection(x1, y1, x2, y2);
			crop.setOptions({
				aspectRatio: '1:1',
				handles: 'corners'
			});
			crop.update();
		});

		$('#thumb_edit').submit(function()
		{
			var s = crop.getSelection();
			var x = Math.round(s.x1 * ratio);
			var y = Math.round(s.y1 * ratio);
			var w = Math.round(s.width * ratio);
			var h = Math.round(s.height * ratio);
			$('input[name="crop_selection"]').val(
				'{"x":' + x + ',"y":' + y + ',"w":' + w + ',"h":' + h + '}'
			);
		});
	});
	
});
