<?php
declare(strict_types = 1);

/**
 * Télécharge un fichier ou un album.
 *
 * @license http://www.gnu.org/licenses/gpl.html
 * @link http://www.igalerie.org/
 */

if (isset($_GET['file']))
{
	$download = TRUE;
	require_once(__DIR__ . '/file.php');
	die;
}

if (isset($_GET['album']))
{
	$gets = ['album', 'key'];
	require_once(__DIR__ . '/includes/prepend.php');

	// Vérification du format des paramètres.
	if (!isset($_GET['album']) || !preg_match('`^\d{1,12}$`i', $_GET['album'])
	 || !isset($_GET['key'])  || !preg_match('`^[a-f0-9]{32}$`', $_GET['key']))
	{
		die(App::httpResponse(400));
	}

	// Vérification de la clé de sécurité.
	if (!App::fileKeyVerify([$_GET['album']], $_GET['key']))
	{
		die(App::httpResponse(403));
	}

	// Connexion à la base de données.
	if (!DB::connect())
	{
		die(App::httpResponse(500));
	}

	// Fonctionnalité activée ?
	Config::getDBParams();
	if (!Config::$params['albums_download'])
	{
		die(App::httpResponse(403));
	}

	// Permission de groupe ?
	if (Config::$params['users'])
	{
		Auth::cookie();
		if (!Auth::$groupPerms['albums_download'])
		{
			die(App::httpResponse(403));
		}
	}

	// Vérification des permissions d'accès à l'album.
	$sql = "SELECT cat_name
			  FROM {categories} AS cat
			 WHERE cat_id = ?
			   AND %s
			   AND %s
			   AND cat_filemtime IS NOT NULL
			   AND cat_downloadable = '1'
			   AND cat_status = '1'";
	$sql = sprintf($sql, SQL::catPerms(), SQL::catPassword());
	if (!DB::execute($sql, $_GET['album']))
	{
		die(App::httpResponse(500));
	}
	if (!$album = DB::fetchRow())
	{
		die(App::httpResponse(403));
	}

	// Récupération des informations utiles des fichiers.
	$sql = "SELECT item_path
			  FROM {items}
			 WHERE cat_id = ?
			   AND item_status = '1'";
	if (!DB::execute($sql, $_GET['album']))
	{
		die(App::httpResponse(500));
	}
	if (!$items = DB::fetchAll())
	{
		die(App::httpResponse(403));
	}

	// Fermeture de la connexion à la base de données.
	DB::disconnect();

	// Fichiers à placer dans l'archive.
	$files = [];
	foreach ($items as &$i)
	{
		$files[] = CONF_ALBUMS_PATH . '/' . $i['item_path'];
	}

	// Envoi de l'archive.
	$items = NULL;
	Zip::archive($album['cat_name'] . '.zip', $files);
	die;
}

require_once(__DIR__ . '/includes/prepend.php');
die(App::httpResponse(400));
?>